@@ -1,6 +1,6 @@
 /********************************************
 files.c
-copyright 2008-2010,2012.  Thomas E. Dickey
+copyright 2008-2012,2016.  Thomas E. Dickey
 copyright 1991-1994,1996.  Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: files.c,v 1.30 2012/12/09 15:07:58 tom Exp $
+ * $MawkId: files.c,v 1.31 2016/09/05 13:46:51 tom Exp $
  *
  * @Log: files.c,v @
  * Revision 1.9  1996/01/14  17:14:10  mike
@@ -154,83 +154,84 @@ output_failed(const char *name)
 PTR
 file_find(STRING * sval, int type)
 {
-    PTR result = 0;
-    FILE_NODE *p;
-    FILE_NODE *q;
+    FILE_NODE *p = file_list;
+    FILE_NODE *q = (FILE_NODE *) 0;
     char *name = sval->str;
 
     TRACE(("file_find(%s, %d)\n", name, type));
-    for (q = 0, p = file_list; p != 0; q = p, p = p->link) {
-	/* search is by name and type */
-	if (strcmp(name, p->name->str) == 0 &&
-	    (p->type == type ||
-	/* no distinction between F_APPEND and F_TRUNC here */
-	     (p->type >= F_APPEND && type >= F_APPEND))) {
-	    if (q != 0) {
-		/* delete from list for move to front */
-		q->link = p->link;
-	    }
-	    break;		/* while loop */
-	}
-    }
+    while (1) {
+	if (!p) {
+	    /* open a new one */
+	    p = alloc_filenode();
 
-    if (!p) {
-	/* open a new one */
-	p = alloc_filenode();
+	    switch (p->type = (short) type) {
+	    case F_TRUNC:
+		if (!(p->ptr = (PTR) tfopen(name, BinMode2("wb", "w"))))
+		    output_failed(name);
+		break;
 
-	switch (p->type = (short) type) {
-	case F_TRUNC:
-	    if (!(p->ptr = (PTR) tfopen(name, BinMode2("wb", "w"))))
-		output_failed(name);
-	    break;
+	    case F_APPEND:
+		if (!(p->ptr = (PTR) tfopen(name, BinMode2("ab", "a"))))
+		    output_failed(name);
+		break;
 
-	case F_APPEND:
-	    if (!(p->ptr = (PTR) tfopen(name, BinMode2("ab", "a"))))
-		output_failed(name);
-	    break;
+	    case F_IN:
+		if (!(p->ptr = (PTR) FINopen(name, 0))) {
+		    free_filenode(p);
+		    return (PTR) 0;
+		}
+		break;
 
-	case F_IN:
-	    p->ptr = (PTR) FINopen(name, 0);
-	    break;
-
-	case PIPE_OUT:
-	case PIPE_IN:
+	    case PIPE_OUT:
+	    case PIPE_IN:
 
 #if    defined(HAVE_REAL_PIPES) || defined(HAVE_FAKE_PIPES)
 
-	    if (!(p->ptr = get_pipe(name, type, &p->pid))) {
-		if (type == PIPE_OUT)
-		    output_failed(name);
-	    }
+		if (!(p->ptr = get_pipe(name, type, &p->pid))) {
+		    if (type == PIPE_OUT)
+			output_failed(name);
+		    else {
+			free_filenode(p);
+			return (PTR) 0;
+		    }
+		}
 #else
-	    rt_error("pipes not supported");
+		rt_error("pipes not supported");
 #endif
-	    break;
+		break;
 
 #ifdef	DEBUG
-	default:
-	    bozo("bad file type");
+	    default:
+		bozo("bad file type");
 #endif
+	    }
+	    /* successful open */
+	    p->name = sval;
+	    sval->ref_cnt++;
+	    break;		/* while loop */
 	}
-    } else if (p->ptr == 0 && type == F_IN) {
-	p->ptr = (PTR) FINopen(name, 0);
-    }
+
+	/* search is by name and type */
+	if (strcmp(name, p->name->str) == 0 &&
+	    (p->ptr != 0) &&
+	    (p->type == type ||
+	/* no distinction between F_APPEND and F_TRUNC here */
+	     (p->type >= F_APPEND && type >= F_APPEND))) {
+	    /* found */
+	    if (!q)		/*at front of list */
+		return p->ptr;
+	    /* delete from list for move to front */
+	    q->link = p->link;
+	    break;		/* while loop */
+	}
+
+	q = p;
+	p = p->link;
+    }				/* end while loop */
 
     /* put p at the front of the list */
-    if (p->ptr == 0) {
-	free_filenode(p);
-    } else {
-	if (p != file_list) {
-	    p->link = file_list;
-	    file_list = p;
-	}
-	/* successful open */
-	p->name = sval;
-	sval->ref_cnt++;
-	TRACE(("-> %p\n", p->ptr));
-	result = p->ptr;
-    }
-    return result;
+    p->link = file_list;
+    return (PTR) (file_list = p)->ptr;
 }
 
 /* Close a file and delete its node from the file_list.
